// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Interfaces/AirbridgeCore.h"

#pragma region Function Key

enum AirbridgeFunctionKey
{
	OnDeeplinkReceived,
	Click_OnSuccess, Click_OnFailure,
	Impression_OnSuccess, Impression_OnFailure,
	FetchDeviceUUID_OnSuccess, FetchDeviceUUID_OnFailure,
	FetchAirbridgeGeneratedUUID_OnSuccess, FetchAirbridgeGeneratedUUID_OnFailure,
};

#pragma endregion 

/**
 *
 */
class AirbridgeAndroidCore : public IAirbridgeCore
{
public:
	AirbridgeAndroidCore();
	virtual ~AirbridgeAndroidCore();

#pragma region Core

	virtual void EnableSDK() override;
	virtual void DisableSDK() override;
	virtual bool IsSDKEnabled() override;

	// virtual void StartInAppPurchaseTracking() override;
	// virtual void StopInAppPurchaseTracking() override;
	// virtual bool IsInAppPurchaseTrackingEnabled() override;
    
#pragma endregion

#pragma region Privacy

	virtual void StartTracking() override;
	virtual void StopTracking() override;
	virtual bool IsTrackingEnabled() override;
    
#pragma endregion

#pragma region Deeplink

	virtual void SetOnDeeplinkReceived(
		const AirbridgeStringParamFunction OnDeeplinkReceived,
		bool UseBroadcast = false
	) override;
    
#pragma endregion

#pragma region Data Collection

	virtual void SetUserID(const FString& Id) override;
	virtual void ClearUserID() override;
	virtual void SetUserEmail(const FString& Email) override;
	virtual void ClearUserEmail() override;
	virtual void SetUserPhone(const FString& Phone) override;
	virtual void ClearUserPhone() override;
	virtual void SetUserAttribute(const FString& Key, const int32 Value) override;
	virtual void SetUserAttribute(const FString& Key, const int64 Value) override;
	virtual void SetUserAttribute(const FString& Key, const float Value) override;
	virtual void SetUserAttribute(const FString& Key, const double Value) override;
	virtual void SetUserAttribute(const FString& Key, const bool Value) override;
	virtual void SetUserAttribute(const FString& Key, const FString& Value) override;
	virtual void RemoveUserAttribute(const FString& Key) override;
	virtual void ClearUserAttributes() override;
	virtual void SetUserAlias(const FString& Key, const FString& Value) override;
	virtual void RemoveUserAlias(const FString& Key) override;
	virtual void ClearUserAlias() override;
	virtual void ClearUser() override;

	virtual void SetDeviceAlias(const FString& Key, const FString& Value) override;
	virtual void RemoveDeviceAlias(const FString& Key) override;
	virtual void ClearDeviceAlias() override;

	virtual void RegisterPushToken(const FString& Token) override;

#pragma endregion 

#pragma region Event
	
	virtual void TrackEvent(
		const FString& Category,
		const UAirbridgeMap* SemanticAttributes = nullptr,
		const UAirbridgeMap* CustomAttributes = nullptr
	) override;

#pragma endregion

#pragma region Placement

	virtual void Click(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction OnSuccess = nullptr,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;
	virtual void Impression(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction OnSuccess = nullptr,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;

#pragma endregion

#pragma region Publication
	
	virtual void FetchDeviceUUID(
		const AirbridgeStringParamFunction OnSuccess,
		const AirbridgeStringParamFunction OnFailure = nullptr,
		bool UseBroadcast = false
	) override;
	virtual void FetchAirbridgeGeneratedUUID(
		const AirbridgeStringParamFunction OnSuccess,
		const AirbridgeStringParamFunction OnFailure = nullptr,
		bool UseBroadcast = false
	) override;
	virtual void SetOnAttributionReceived(
		const AirbridgeMapParamFunction OnAttributionReceived,
		bool UseBroadcast = false
	) override;

#pragma endregion 

#pragma region Hybrid

	virtual FString CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript) override;
	virtual void HandleWebInterfaceCommand(const FString& Command) override;

#pragma endregion

private:
	TMap<AirbridgeFunctionKey, TMap<FString, AirbridgeStringParamFunction>>	AirbridgeStringParamFunctionMap;
	TMap<AirbridgeFunctionKey, TMap<FString, AirbridgeVoidParamFunction>>	AirbridgeVoidParamFunctionMap;

	void SetAirbridgeStringParamFunctionMap(
		const AirbridgeFunctionKey Key,
		const FString& UUID,
		const AirbridgeStringParamFunction& Function
	);
	void SetAirbridgeVoidParamFunctionMap(
		const AirbridgeFunctionKey Key,
		const FString& UUID,
		const AirbridgeVoidParamFunction& Function
	);

#pragma region Deeplink
	
public:
	static FString StartDeeplink;
	void ProcessHandleDeeplink(const FString& Uri);
	void ProcessHandleDeferredDeeplink(const FString& Uri);

private:
	bool DidSetUnrealBroadcastDeeplink = false;
	void HandleDeeplinkOnSuccess(const FString& Uri);
	
#pragma endregion 

public:
	void ProcessClickOnSuccess(const FString& MethodUUID);
	void ProcessClickOnFailure(const FString& MethodUUID, const FString& Error);

	void ProcessImpressionOnSuccess(const FString& MethodUUID);
	void ProcessImpressionOnFailure(const FString& MethodUUID, const FString& Error);

	void UnrealBroadcastDeviceUUID(const FString& DeviceUUID);
	void ProcessFetchDeviceUUIDOnSuccess(const FString& MethodUUID, const FString& DeviceUUID);
	void ProcessFetchDeviceUUIDOnFailure(const FString& MethodUUID, const FString& Error);

	void UnrealBroadcastAirbridgeGeneratedUUID(const FString& AirbridgeGeneratedUUID);
	void ProcessFetchAirbridgeGeneratedUUIDOnSuccess(const FString& MethodUUID, const FString& AirbridgeGeneratedUUID);
	void ProcessFetchAirbridgeGeneratedUUIDOnFailure(const FString& MethodUUID, const FString& Error);

#pragma region Attribution
	
public:
	static TMap<FString, FString> ReceivedAttribution;
	void ProcessOnAttributionResultReceived(const TMap<FString, FString>& Data);

private:
	bool DidSetUnrealBroadcastAttribution = false;
	AirbridgeMapParamFunction AirbridgeOnAttributionResultReceiveListener = nullptr;
	void HandleOnAttributionReceived();

#pragma endregion 
};