// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "AirbridgeUnreal.h"

#include "Kismet/BlueprintFunctionLibrary.h"
#include "AirbridgeBlueprint.generated.h"

UCLASS()
class UAirbridgeBlueprint : public UBlueprintFunctionLibrary
{
	GENERATED_UCLASS_BODY()

#pragma region Core

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	static void EnableSDK();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	static void DisableSDK();
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	static bool IsSDKEnabled();

	// UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	// static void StartInAppPurchaseTracking();
	//
	// UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	// static void StopInAppPurchaseTracking();
	//
	// UFUNCTION(BlueprintCallable, Category = "Airbridge|Core")
	// static bool IsInAppPurchaseTrackingEnabled();

#pragma endregion

#pragma region Privacy

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Privacy")
	static void StartTracking();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Privacy")
	static void StopTracking();
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Privacy")
	static bool IsTrackingEnabled();

#pragma endregion

#pragma region Deeplink

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Deeplink")
	static void SetOnDeeplinkReceived();
    
#pragma endregion

#pragma region Data Collection

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserID(const FString& Id);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUserID();
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserEmail(const FString& Email);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUserEmail();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserPhone(const FString& Phone);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUserPhone();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserIntAttribute(const FString& Key, const int32 Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserLongAttribute(const FString& Key, const int64 Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserFloatAttribute(const FString& Key, const float Value);
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserBooleanAttribute(const FString& Key, const bool Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserStringAttribute(const FString& Key, const FString& Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void RemoveUserAttribute(const FString& Key);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUserAttributes();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void SetUserAlias(const FString& Key, const FString& Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void RemoveUserAlias(const FString& Key);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUserAlias();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|User")
	static void ClearUser();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|Device")
	static void SetDeviceAlias(const FString& Key, const FString& Value);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|Device")
	static void RemoveDeviceAlias(const FString& Key);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection|Device")
	static void ClearDeviceAlias();

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Data Collection")
	static void RegisterPushToken(const FString& Token);

#pragma endregion 

#pragma region Event

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Event")
	static void TrackEvent(
		UPARAM(DisplayName = "Category") const FString& EventCategory, 
		const UAirbridgeMap* SemanticAttributes = nullptr,
		const UAirbridgeMap* CustomAttributes = nullptr
	);

#pragma endregion
	
#pragma region Placement

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Placement")
	static void Click(const FString& TrackingLink);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Placement")
	static void Impression(const FString& TrackingLink);

#pragma endregion

#pragma region Publication

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Publication")
	static void FetchDeviceUUID();
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Publication")
	static void FetchAirbridgeGeneratedUUID();
	
	UFUNCTION(BlueprintCallable, Category = "Airbridge|Publication")
	static void SetOnAttributionReceived();

#pragma endregion 

#pragma region Hybrid

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Hybrid")
	static FString CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript);

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Hybrid")
	static void HandleWebInterfaceCommand(const FString& Command);

#pragma endregion

	UFUNCTION(BlueprintCallable, Category = "Airbridge|Uninstall Tracking")
	static bool IsUninstallTrackingNotification(const TMap<FString, FString>& Data);
};
