// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "AirbridgeCallbacks.generated.h"

USTRUCT(Blueprintable)
struct FAirbridgeAttribution
{
	GENERATED_BODY()

	UPROPERTY(BlueprintReadOnly)
	TMap<FString, FString> Data;
};

DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnDeeplinkReceived, const FString&, Url);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnAttributionReceived, const FAirbridgeAttribution&, AttributionResult);

DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnUUIDFetched, const FString&, UUID);

UCLASS( ClassGroup=("Airbridge Unreal SDK"), meta=(BlueprintSpawnableComponent) )
class AIRBRIDGEUNREAL_API UAirbridgeCallbacks : public UActorComponent
{
	GENERATED_BODY()

public:	
	UAirbridgeCallbacks(const FObjectInitializer& ObjectInitializer);
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, Category = "Airbridge Unreal SDK",
		meta=(ToolTip="This event is triggered when a deeplink is received."))
	FOnDeeplinkReceived OnDeeplinkReceived;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, Category = "Airbridge Unreal SDK",
		meta=(ToolTip="This event is triggered when an attribution result data is received."))
	FOnAttributionReceived OnAttributionReceived;

	UPROPERTY(BlueprintAssignable, BlueprintCallable,
		Category = "Airbridge Unreal SDK", DisplayName = "On Device UUID Fetched",
		meta=(ToolTip="This event is triggered when the Device UUID is fetched."))
	FOnUUIDFetched OnDeviceUUIDFetched;

	UPROPERTY(BlueprintAssignable, BlueprintCallable,
		Category = "Airbridge Unreal SDK", DisplayName = "On Airbridge Generated UUID Fetched",
		meta=(ToolTip="This event is triggered when the Airbridge Generated UUID is fetched."))
	FOnUUIDFetched OnAirbridgeGeneratedUUIDFetched;
};