// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Airbridge/Container/AirbridgeContainer.h"
#include "Interfaces/AirbridgeCore.h"

#define SUPPORTED_PLATFORM (PLATFORM_ANDROID || PLATFORM_IOS)

class AIRBRIDGEUNREAL_API FAirbridge
{
public:

#pragma region Core

	static void EnableSDK();
	static void DisableSDK();
	static bool IsSDKEnabled();

	// static void StartInAppPurchaseTracking();
	// static void StopInAppPurchaseTracking();
	// static bool IsInAppPurchaseTrackingEnabled();
    
#pragma endregion
	
#pragma region Privacy

	static void StartTracking();
	static void StopTracking();
	static bool IsTrackingEnabled();
    
#pragma endregion

#pragma region Deeplink
	
	static void SetOnDeeplinkReceived(const AirbridgeStringParamFunction& OnDeeplinkReceived);
    
#pragma endregion

#pragma region Data Collection

	static void SetUserID(const FString& Id);
	static void ClearUserID();
	static void SetUserEmail(const FString& Email);
	static void ClearUserEmail();
	static void SetUserPhone(const FString& Phone);
	static void ClearUserPhone();
	static void SetUserAttribute(const FString& Key, const int32 Value);
	static void SetUserAttribute(const FString& Key, const int64 Value);
	static void SetUserAttribute(const FString& Key, const float Value);
	static void SetUserAttribute(const FString& Key, const double Value);
	static void SetUserAttribute(const FString& Key, const bool Value);
	static void SetUserAttribute(const FString& Key, const FString& Value);

	static void RemoveUserAttribute(const FString& Key);
	static void ClearUserAttributes();
	static void SetUserAlias(const FString& Key, const FString& Value);
	static void RemoveUserAlias(const FString& Key);
	static void ClearUserAlias();
	static void ClearUser();
	
	static void SetDeviceAlias(const FString& Key, const FString& Value);
	static void RemoveDeviceAlias(const FString& Key);
	static void ClearDeviceAlias();

	static void RegisterPushToken(const FString& Token);

#pragma endregion 

#pragma region Event
	
	static void TrackEvent(
		const FString& Category,
		const UAirbridgeMap* SemanticAttributes = nullptr,
		const UAirbridgeMap* CustomAttributes = nullptr
	);

#pragma endregion

#pragma region Placement

	static void Click(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction& OnSuccess = nullptr,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);
	static void Impression(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction& OnSuccess = nullptr,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);

#pragma endregion

#pragma region Publication
	
	static void FetchDeviceUUID(
		const AirbridgeStringParamFunction& OnSuccess,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);
	static void FetchAirbridgeGeneratedUUID(
		const AirbridgeStringParamFunction& OnSuccess,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);
	static void SetOnAttributionReceived(const AirbridgeMapParamFunction& OnAttributionReceived);

#pragma endregion 

#pragma region Hybrid

	static FString CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript);
	static void HandleWebInterfaceCommand(const FString& Command);

#pragma endregion

	static bool IsUninstallTrackingNotification(const TMap<FString, FString>& Data);
};
